@@ -1,10 +1,10 @@
 # coding: utf-8
 
 
-from collections import OrderedDict
+
 from collections import abc
 
-from sympy.core import Basic, Symbol
+from sympy.core import Basic, Symbol, Expr
 from sympy.core.containers import Tuple
 from sympy.tensor import IndexedBase
 
@@ -100,16 +100,10 @@ class InteriorDomain(BasicDomain):
 
     def _sympystr(self, printer):
         sstr = printer.doprint
-        if self.mapping:
-            return '{}({})'.format(sstr(self.mapping.name), sstr(self.name))
-        else:
-            return '{}'.format(sstr(self.name))
+        return '{}'.format(sstr(self.name))
 
     def todict(self):
-        name   = str(self.name)
-        d = {'name': name}
-
-        return OrderedDict(sorted(d.items()))
+        return {'name': str(self.name)}
 
 
 #==============================================================================
@@ -309,6 +303,43 @@ class Boundary(BasicDomain):
     def dim(self):
         return self.domain.dim
 
+    @property
+    def adjacent_boundaries(self):
+        boundaries = [a for a in self.domain.boundary if a.axis !=self.axis]
+        return Union(*boundaries)
+
+    def rotate(self, *directions):
+        assert len(directions) == self.dim-1
+
+        if self.dim == 2:
+            if directions[0] == 1:
+                return self
+            elif directions[0] == -1:
+                return self.domain.get_boundary(axis=self.axis, ext=-self.ext)
+
+            else:
+                raise TypeError('must be int')
+
+        raise NotImplementedError('only 2d case is available')
+
+    def join(self, boundary, direction=None):
+        from sympde.topology.mapping import InterfaceMapping
+        # TODO be careful with '|' in psydac
+        if self.mapping and boundary.mapping:
+            int_map            = InterfaceMapping(self.mapping , boundary.mapping)
+            a,b                = self.logical_domain, boundary.logical_domain
+            l_name             = '{l}|{r}'.format(l=a.domain.name, r=b.domain.name)
+            int_logical_domain = Interface(l_name, a,b, direction=direction)
+        else:
+            int_map            = None
+            int_logical_domain = None
+
+        name               = '{l}|{r}'.format(l=self.domain.name, r=boundary.domain.name)
+        interface = Interface(name, self, boundary,
+                              mapping=int_map,
+                              logical_domain=int_logical_domain, direction=direction)
+        return interface
+
     def _sympystr(self, printer):
         sstr = printer.doprint
         return '{}_{}'.format(sstr(self.domain),sstr(self.name))
@@ -320,17 +351,84 @@ class Boundary(BasicDomain):
         return Union(self, other)
 
     def todict(self):
-        domain = str(self.domain.name)
-        name   = str(self.name)
-        axis   = str(self.axis)
-        ext    = str(self.ext)
+        return {'axis'  : str(self.axis),
+                'ext'  : str(self.ext),
+                'name' : str(self.name),
+                'patch': str(self.domain.name)}
 
-        d = {'patch': domain,
-             'name':  name,
-             'axis':  axis,
-             'ext':   ext}
+#==============================================================================
+class CornerBoundary(BasicDomain):
+    """
+    Represents an undefined corner over a domain in 2D.
 
-        return OrderedDict(sorted(d.items()))
+    """
+    def __new__(cls, *boundaries):
+        assert all(isinstance(i, Boundary) for i in boundaries)
+        assert all(i.domain==boundaries[0].domain for i in boundaries)
+
+        boundaries = sorted(boundaries, key=lambda x:x.axis)
+        obj = Basic.__new__(cls, *boundaries)
+        obj._domain = boundaries[0].domain
+        return obj
+
+    @property
+    def boundaries(self):
+        return self._args
+
+    @property
+    def domain(self):
+        return self._domain
+
+    @property
+    def coordinates(self):
+        coords = [None]*self.domain.dim
+        for b in self.boundaries:
+            coords[b.axis] = (b.ext + 1)//2
+        return tuple(coords)
+
+    @property
+    def logical_domain(self):
+        boundaries = [a.logical_domain for a in self.boundaries]
+        if boundaries[0]:
+            return CornerBoundary(*boundaries)
+        else:
+            return None
+
+    def _sympystr(self, printer):
+        sstr = printer.doprint
+        boundaries = ', '.join(sstr(b) for b in self.boundaries)
+        return 'CornerBoundary({})'.format(boundaries)
+
+#==============================================================================
+class CornerInterface(BasicDomain):
+    """
+    Represents a shared corner over multiple patches in 2D.
+
+    """
+    def __new__(cls, *corners):
+        assert all(isinstance(i, CornerBoundary) for i in corners)
+        corners = sorted(corners, key=lambda x:x.domain.name)
+        return Basic.__new__(cls, *corners)
+
+    @property
+    def corners(self):
+        return self._args
+
+    @property
+    def logical_domain(self):
+        corners = [a.logical_domain for a in self.corners]
+        if corners[0]:
+            return CornerInterface(*corners)
+        else:
+            return None
+
+    def __len__(self):
+        return len(self.corners)
+
+    def _sympystr(self, printer):
+        sstr = printer.doprint
+        corners = ', '.join(sstr(b) for b in self.corners)
+        return 'CornerInterface({})'.format(corners)
 
 #==============================================================================
 class Interface(BasicDomain):
@@ -340,7 +438,7 @@ class Interface(BasicDomain):
     Examples
 
     """
-    def __new__(cls, name, bnd_minus, bnd_plus, mapping=None, logical_domain=None):
+    def __new__(cls, name, bnd_minus, bnd_plus, mapping=None, logical_domain=None, direction=None):
 
         if not isinstance(name     , str    ): raise TypeError(name)
         if not isinstance(bnd_minus, Boundary): raise TypeError(bnd_minus)
@@ -354,7 +452,7 @@ class Interface(BasicDomain):
                mapping is not None and logical_domain is not None
 
         assert bnd_minus.axis == bnd_plus.axis
-        obj = Basic.__new__(cls, name, bnd_minus, bnd_plus)
+        obj = Basic.__new__(cls, name, bnd_minus, bnd_plus, direction)
         obj._mapping        = mapping
         obj._logical_domain = logical_domain
         return obj
@@ -375,6 +473,10 @@ class Interface(BasicDomain):
     def plus(self):
         return self.args[2]
 
+    @property
+    def direction(self):
+        return self.args[3]
+
     @property
     def axis(self):
         return self.plus.axis
@@ -389,22 +491,8 @@ class Interface(BasicDomain):
 
     def _sympystr(self, printer):
         sstr = printer.doprint
-
-#        name  = self.name
-#        minus = self.minus
-#        plus  = self.plus
-#        minus = '{domain}.{bnd}'.format( domain = sstr(minus.domain),
-#                                         bnd    = sstr(minus) )
-#        plus = '{domain}.{bnd}'.format( domain = sstr(plus.domain),
-#                                         bnd    = sstr(plus) )
-#        pattern = 'Interface( {name}; {minus}, {plus} )'
-#        return pattern.format( name  = sstr(self.name),
-#                               minus = minus,
-#                               plus = plus )
-
         return '{}'.format(sstr(self.name))
 
-
 #==============================================================================
 class Edge(object):
     def __init__(self, name):
@@ -417,7 +505,6 @@ class Edge(object):
     def __lt__(self, other):
         return self.name.__lt__(other.name)
 
-
 #==============================================================================
 class Connectivity(abc.Mapping):
     _patches  = []
@@ -426,7 +513,7 @@ class Connectivity(abc.Mapping):
         if data is None:
             data = {}
         else:
-            assert( isinstance( data, (dict, OrderedDict)) )
+            assert isinstance( data, dict )
             for k,v in data.items():
                 assert( isinstance( k, str ) )
                 assert( isinstance(v, Interface) )
@@ -439,7 +526,7 @@ class Connectivity(abc.Mapping):
     @property
     def interfaces(self):
         ls = []
-        data = OrderedDict(sorted(self._data.items()))
+        data = dict(sorted(self._data.items()))
         for _,v in data.items():
             ls.append(v)
         return Union(*ls)
@@ -447,10 +534,10 @@ class Connectivity(abc.Mapping):
     def todict(self):
         # ... create the connectivity
         connectivity = {}
-        data = OrderedDict(sorted(self._data.items()))
+        data = dict(sorted(self._data.items()))
         for name, v in data.items():
             connectivity[name] = [v.minus.todict(), v.plus.todict()]
-        connectivity = OrderedDict(sorted(connectivity.items()))
+        connectivity = dict(sorted(connectivity.items()))
         # ...
 
         return connectivity
